@@ -1,7 +1,7 @@
 /*
  * client.c	Read clients into memory.
  *
- * Version:     $Id$
+ * Version:     $Id: 5567546826ec327790d1968fcf1972fcba1b58d2 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id$")
+RCSID("$Id: 5567546826ec327790d1968fcf1972fcba1b58d2 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/rad_assert.h>
@@ -63,6 +63,8 @@ static fr_fifo_t	*deleted_clients = NULL;
  */
 void client_free(RADCLIENT *client)
 {
+	if (!client) return;
+
 #ifdef WITH_DYNAMIC_CLIENTS
 	if (client->dynamic == 2) {
 		time_t now;
@@ -270,17 +272,60 @@ int client_add(RADCLIENT_LIST *clients, RADCLIENT *client)
 	}
 
 	/*
-	 *	If "clients" is NULL, it means add to the global list.
+	 *	If "clients" is NULL, it means add to the global list,
+	 *	unless we're trying to add it to a virtual server...
 	 */
 	if (!clients) {
-		/*
-		 *	Initialize it, if not done already.
-		 */
-		if (!root_clients) {
-			root_clients = clients_init();
-			if (!root_clients) return 0;
+		if (client->server != NULL) {
+			CONF_SECTION *cs;
+			CONF_SECTION *subcs;
+
+			cs = cf_section_sub_find_name2(mainconfig.config,
+						       "server", client->server);
+			if (!cs) {
+				radlog(L_ERR, "Failed to find virtual server %s",
+				       client->server);
+				return 0;
+			}
+
+			/*
+			 *	If this server has no "listen" section, add the clients
+			 *	to the global client list.
+			 */
+			subcs = cf_section_sub_find(cs, "listen");
+			if (!subcs) goto global_clients;
+
+			/*
+			 *	If the client list already exists, use that.
+			 *	Otherwise, create a new client list.
+			 */
+			clients = cf_data_find(cs, "clients");
+			if (!clients) {
+				clients = clients_init();
+				if (!clients) {
+					radlog(L_ERR, "Out of memory");
+					return 0;
+				}
+
+				if (cf_data_add(cs, "clients", clients, (void *) clients_free) < 0) {
+					radlog(L_ERR, "Failed to associate clients with virtual server %s",
+					       client->server);
+					clients_free(clients);
+					return 0;
+				}
+			}
+
+		} else {
+		global_clients:
+			/*
+			 *	Initialize the global list, if not done already.
+			 */
+			if (!root_clients) {
+				root_clients = clients_init();
+				if (!root_clients) return 0;
+			}
+			clients = root_clients;
 		}
-		clients = root_clients;
 	}
 
 	if ((client->prefix < 0) || (client->prefix > 128)) {
@@ -403,7 +448,11 @@ int client_add(RADCLIENT_LIST *clients, RADCLIENT *client)
 #ifdef WITH_DYNAMIC_CLIENTS
 void client_delete(RADCLIENT_LIST *clients, RADCLIENT *client)
 {
-	if (!clients || !client) return;
+	if (!client) return;
+
+	if (!clients) clients = root_clients;
+
+	if (!client->dynamic) return;
 
 	rad_assert((client->prefix >= 0) && (client->prefix <= 128));
 
@@ -531,6 +580,8 @@ static const CONF_PARSER client_config[] = {
 	  offsetof(RADCLIENT, client_server), 0, NULL },
 	{ "lifetime",  PW_TYPE_INTEGER,
 	  offsetof(RADCLIENT, lifetime), 0, NULL },
+	{ "rate_limit",  PW_TYPE_BOOLEAN,
+	  offsetof(RADCLIENT, rate_limit), 0, NULL },
 #endif
 
 #ifdef WITH_COA
@@ -740,8 +791,8 @@ static RADCLIENT *client_parse(CONF_SECTION *cs, int in_server)
 							   HOME_TYPE_COA);
 		}
 		if (!c->coa_pool && !c->coa_server) {
-			client_free(c);
 			cf_log_err(cf_sectiontoitem(cs), "No such home_server or home_server_pool \"%s\"", c->coa_name);
+			client_free(c);
 			return NULL;
 		}
 	}
@@ -868,6 +919,7 @@ RADCLIENT_LIST *clients_parse_section(CONF_SECTION *section)
 					cf_log_err(cf_sectiontoitem(cs),
 						   "Failed reading client file \"%s\"", buf2);
 					client_free(c);
+					closedir(dir);
 					return NULL;
 				}
 
@@ -877,9 +929,11 @@ RADCLIENT_LIST *clients_parse_section(CONF_SECTION *section)
 				if (!client_validate(clients, c, dc)) {
 					
 					client_free(c);
+					closedir(dir);
 					return NULL;
 				}
 			} /* loop over the directory */
+			closedir(dir);
 		}
 #endif /* HAVE_DIRENT_H */
 #endif /* WITH_DYNAMIC_CLIENTS */
@@ -1047,7 +1101,11 @@ RADCLIENT *client_create(RADCLIENT_LIST *clients, REQUEST *request)
 		case PW_TYPE_STRING_PTR:
 			p = (char **) ((char *) c + dynamic_config[i].offset);
 			if (*p) free(*p);
-			*p = strdup(vp->vp_strvalue);
+			if (vp->vp_strvalue[0]) {
+				*p = strdup(vp->vp_strvalue);
+			} else {
+				*p = NULL;
+			}
 			break;
 
 		case PW_TYPE_BOOLEAN:
@@ -1079,6 +1137,13 @@ RADCLIENT *client_create(RADCLIENT_LIST *clients, REQUEST *request)
 		goto error;
 	}
 
+	if (!c->secret || !*c->secret) {
+		DEBUG("- Cannot add client %s: No secret was specified.",
+		      ip_ntoh(&request->packet->src_ipaddr,
+			      buffer, sizeof(buffer)));
+		goto error;
+	}
+
 	if (!client_validate(clients, request->client, c)) {
 		return NULL;
 	}
@@ -1100,8 +1165,15 @@ RADCLIENT *client_read(const char *filename, int in_server, int flag)
 
 	cs = cf_file_read(filename);
 	if (!cs) return NULL;
+	
+	cs = cf_section_sub_find(cs, "client");
+	if (!cs) {
+		radlog(L_ERR, "No \"client\" section found in client file");
+		return NULL;
+	}
 
-	c = client_parse(cf_section_sub_find(cs, "client"), in_server);
+	c = client_parse(cs, in_server);
+	if (!c) return NULL;
 
 	p = strrchr(filename, FR_DIR_SEP);
 	if (p) {
@@ -1122,8 +1194,6 @@ RADCLIENT *client_read(const char *filename, int in_server, int flag)
 		return NULL;
 	}
 
-
-
 	return c;
 }
 #endif
