@@ -1,7 +1,7 @@
 /*
  * event.c	Server event handling
  *
- * Version:	$Id: ca3edf0ed99ab7444714fae46d734c41b563537b $
+ * Version:	$Id: 80174316c718eee025e497f2a1ed81be1fb91a54 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: ca3edf0ed99ab7444714fae46d734c41b563537b $")
+RCSID("$Id: 80174316c718eee025e497f2a1ed81be1fb91a54 $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -905,7 +905,7 @@ static void check_for_zombie_home_server(REQUEST *request)
 		return;
 	}
 
-	mark_home_server_dead(home, &request->when);
+	mark_home_server_dead(home, &when);
 }
 
 static int proxy_to_virtual_server(REQUEST *request);
@@ -2475,6 +2475,16 @@ static void request_post_handler(REQUEST *request)
 			request->reply->code = PW_AUTHENTICATION_REJECT;
 		}
 
+		/*
+		 *	Do post-auth.  If it returns reject, then
+		 *	run Post-Auth-Type Reject.
+		 */
+		if (request->reply->code == PW_AUTHENTICATION_ACK) {
+			if (rad_postauth(request) == RLM_MODULE_REJECT) {
+				request->reply->code = PW_AUTHENTICATION_REJECT;
+			}
+		}
+		
 		/*
 		 *	Run rejected packets through
 		 *
@@ -2510,10 +2520,6 @@ static void request_post_handler(REQUEST *request)
 			}
 		}
 		
-		if (request->reply->code == PW_AUTHENTICATION_ACK) {
-			rad_postauth(request);
-		}
-		
 		/* FALL-THROUGH */
 
 #ifdef WITH_COA
