@@ -1,7 +1,7 @@
 /*
  * util.c	Various utility functions.
  *
- * Version:     $Id$
+ * Version:     $Id: c717fdd0fe52475e79d44809f5d7c1401fbdc4cd $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id$")
+RCSID("$Id: c717fdd0fe52475e79d44809f5d7c1401fbdc4cd $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/rad_assert.h>
@@ -49,6 +49,7 @@ void (*reset_signal(int signo, void (*func)(int)))(int)
 #ifdef HAVE_SIGACTION
 	struct sigaction act, oact;
 
+	memset(&act, 0, sizeof(act));
 	act.sa_handler = func;
 	sigemptyset(&act.sa_mask);
 	act.sa_flags = 0;
@@ -197,6 +198,12 @@ void request_free(REQUEST **request_ptr)
 
 	request = *request_ptr;
 
+	rad_assert(!request->in_request_hash);
+#ifdef WITH_PROXY
+	rad_assert(!request->in_proxy_hash);
+#endif
+	rad_assert(!request->ev);
+
 	if (request->packet)
 		rad_free(&request->packet);
 
@@ -238,6 +245,18 @@ void request_free(REQUEST **request_ptr)
 		request->root = NULL;
 	}
 
+#ifdef WITH_COA
+	if (request->coa) {
+		request->coa->parent = NULL;
+		rad_assert(request->coa->ev == NULL);
+		request_free(&request->coa);
+	}
+
+	if (request->parent && (request->parent->coa == request)) {
+		request->parent->coa = NULL;
+	}
+#endif
+
 #ifndef NDEBUG
 	request->magic = 0x01020304;	/* set the request to be nonsense */
 #endif
@@ -317,6 +336,105 @@ int rad_mkdir(char *directory, int mode)
 	return mkdir(directory, mode);
 }
 
+/** Escapes the raw string such that it should be safe to use as part of a file path
+ *
+ * This function is designed to produce a string that's still readable but portable
+ * across the majority of file systems.
+ *
+ * For security reasons it cannot remove characters from the name, and must not allow
+ * collisions to occur between different strings.
+ *
+ * With that in mind '-' has been chosen as the escape character, and will be double
+ * escaped '-' -> '--' to avoid collisions.
+ *
+ * Escaping should be reversible if the original string needs to be extracted.
+ *
+ * @note function takes additional arguments so that it may be used as an xlat escape
+ *	function but it's fine to call it directly.
+ *
+ * @note OSX/Unix/NTFS/VFAT/vfat have a max filename size of 255 bytes.
+ *
+ * @param request Current request (may be NULL).
+ * @param out Output buffer.
+ * @param outlen Size of the output buffer.
+ * @param in string to escape.
+ * @param arg Context arguments (unused, should be NULL).
+ */
+size_t rad_filename_escape(char *out, size_t outlen, char const *in)
+{
+	size_t freespace = outlen;
+
+	while (in[0]) {
+		size_t utf8_len;
+
+		/*
+		 *	Encode multibyte UTF8 chars
+		 */
+		utf8_len = fr_utf8_char((uint8_t const *) in);
+		if (utf8_len > 1) {
+			if (freespace <= (utf8_len * 3)) break;
+
+			switch (utf8_len) {
+			case 2:
+				snprintf(out, freespace, "-%x-%x", in[0], in[1]);
+				break;
+
+			case 3:
+				snprintf(out, freespace, "-%x-%x-%x", in[0], in[1], in[2]);
+				break;
+
+			case 4:
+				snprintf(out, freespace, "-%x-%x-%x-%x", in[0], in[1], in[2], in[3]);
+				break;
+			}
+
+			freespace -= (utf8_len * 3);
+			out += (utf8_len * 3);
+			in += utf8_len;
+
+			continue;
+		}
+
+		/*
+		 *	Safe chars
+		 */
+		if (((in[0] >= 'A') && (in[0] <= 'Z')) ||
+		    ((in[0] >= 'a') && (in[0] <= 'z')) ||
+		    ((in[0] >= '0') && (in[0] <= '9')) ||
+		    (in[0] == '_') || (in[0] == '.')) {
+		    	if (freespace <= 1) break;
+
+		 	*out++ = *in;
+		 	in++;
+		 	freespace--;
+		 	continue;
+		}
+
+		if (freespace <= 2) break;
+
+		/*
+		 *	Double escape '-' (like \\)
+		 */
+		if (in[0] == '-') {
+			*out++ = '-';
+			*out++ = '-';
+
+			freespace -= 2;
+			in++;
+			continue;
+		}
+
+		/*
+		 *	Unsafe chars
+		 */
+		*out++ = '-';
+		fr_bin2hex((uint8_t *)in++, out, 1);
+		out += 2;
+		freespace -= 3;
+	}
+	*out = '\0';
+	return outlen - freespace;
+}
 
 /*
  *	Module malloc() call, which does stuff if the malloc fails.
@@ -374,7 +492,7 @@ REQUEST *request_alloc(void)
 	request->options = RAD_REQUEST_OPTION_NONE;
 
 	request->module = "";
-	request->component = "";
+	request->component = "<core>";
 	if (debug_flag) request->radlog = radlog_request;
 
 	return request;
@@ -395,7 +513,7 @@ REQUEST *request_alloc_fake(REQUEST *request)
 
   fake->number = request->number;
 #ifdef HAVE_PTHREAD_H
-  fake->child_pid = request->child_pid;
+  fake->thread_id = request->thread_id;
 #endif
   fake->parent = request;
   fake->root = request->root;
@@ -410,7 +528,7 @@ REQUEST *request_alloc_fake(REQUEST *request)
    */
   fake->server = request->server;
 
-  fake->packet = rad_alloc(0);
+  fake->packet = rad_alloc(1);
   if (!fake->packet) {
 	  request_free(&fake);
 	  return NULL;
@@ -442,7 +560,7 @@ REQUEST *request_alloc_fake(REQUEST *request)
   fake->packet->id = fake->number & 0xff;
   fake->packet->code = request->packet->code;
   fake->timestamp = request->timestamp;
- 
+
   /*
    *	Required for new identity support
    */
@@ -468,6 +586,25 @@ REQUEST *request_alloc_fake(REQUEST *request)
   return fake;
 }
 
+#ifdef WITH_COA
+REQUEST *request_alloc_coa(REQUEST *request)
+{
+	if (!request || request->coa) return NULL;
+
+	/*
+	 *	Originate CoA requests only when necessary.
+	 */
+	if ((request->packet->code != PW_AUTHENTICATION_REQUEST) &&
+	    (request->packet->code != PW_ACCOUNTING_REQUEST)) return NULL;
+
+	request->coa = request_alloc_fake(request);
+	request->coa->packet->code = 0; /* unknown, as of yet */
+	request->coa->child_state = REQUEST_RUNNING;
+	request->coa->proxy = rad_alloc(0);
+
+	return request->coa;
+}
+#endif
 
 /*
  *	Copy a quoted string.
@@ -495,6 +632,33 @@ int rad_copy_string(char *to, const char *from)
 	return length;
 }
 
+/*
+ *	Copy a quoted string but without the quotes. The length
+ *	returned is the number of chars written; the number of
+ *	characters consumed is 2 more than this.
+ */
+int rad_copy_string_bare(char *to, const char *from)
+{
+	int length = 0;
+	char quote = *from;
+
+	from++;
+	while (*from && (*from != quote)) {
+		if (*from == '\\') {
+			*(to++) = *(from++);
+			length++;
+		}
+		*(to++) = *(from++);
+		length++;
+	}
+
+	if (*from != quote) return -1; /* not properly quoted */
+
+	*to = '\0';
+
+	return length;
+}
+
 
 /*
  *	Copy a %{} string.
@@ -515,6 +679,7 @@ int rad_copy_variable(char *to, const char *from)
 			if (sublen < 0) return sublen;
 			from += sublen;
 			to += sublen;
+			length += sublen;
 			break;
 
 		case '}':	/* end of variable expansion */
@@ -539,11 +704,10 @@ int rad_copy_variable(char *to, const char *from)
 				from += sublen;
 				to += sublen;
 				length += sublen;
+				break;
 			} /* else FIXME: catch %%{ ?*/
 
 			/* FALL-THROUGH */
-			break;
-
 		default:
 			*(to++) = *(from++);
 			length++;
@@ -558,3 +722,158 @@ int rad_copy_variable(char *to, const char *from)
 	return -1;
 }
 
+/*
+ * Split a string into words, xlat each one and write into argv array.
+ * Return argc or -1 on failure.
+ */
+
+int rad_expand_xlat(REQUEST *request, const char *cmd,
+		    int max_argc, const char *argv[], int can_fail,
+		    size_t argv_buflen, char *argv_buf)
+{
+	const char *from;
+	char *to;
+	int argc = -1;
+	int i;
+	int left;
+
+	if (strlen(cmd) > (argv_buflen - 1)) {
+		radlog(L_ERR|L_CONS, "rad_expand_xlat: Command line is too long");
+		return -1;
+	}
+
+	/*
+	 *	Check for bad escapes.
+	 */
+	if (cmd[strlen(cmd) - 1] == '\\') {
+		radlog(L_ERR|L_CONS, "rad_expand_xlat: Command line has final backslash, without a following character");
+		return -1;
+	}
+
+	strlcpy(argv_buf, cmd, argv_buflen);
+
+	/*
+	 *	Split the string into argv's BEFORE doing radius_xlat...
+	 */
+	from = cmd;
+	to = argv_buf;
+	argc = 0;
+	while (*from) {
+		int length;
+
+		/*
+		 *	Skip spaces.
+		 */
+		if ((*from == ' ') || (*from == '\t')) {
+			from++;
+			continue;
+		}
+
+		argv[argc] = to;
+		argc++;
+
+		if (argc >= (max_argc - 1)) break;
+
+		/*
+		 *	Copy the argv over to our buffer.
+		 */
+		while (*from && (*from != ' ') && (*from != '\t')) {
+			if (to >= argv_buf + argv_buflen - 1) {
+				radlog(L_ERR|L_CONS, "rad_expand_xlat: Ran out of space in command line");
+				return -1;
+			}
+
+			switch (*from) {
+			case '"':
+			case '\'':
+				length = rad_copy_string_bare(to, from);
+				if (length < 0) {
+					radlog(L_ERR|L_CONS, "rad_expand_xlat: Invalid string passed as argument");
+					return -1;
+				}
+				from += length+2;
+				to += length;
+				break;
+
+			case '%':
+				if (from[1] == '{') {
+					*(to++) = *(from++);
+
+					length = rad_copy_variable(to, from);
+					if (length < 0) {
+						radlog(L_ERR|L_CONS, "rad_expand_xlat: Invalid variable expansion passed as argument");
+						return -1;
+					}
+					from += length;
+					to += length;
+				} else { /* FIXME: catch %%{ ? */
+					*(to++) = *(from++);
+				}
+				break;
+
+			case '\\':
+				if (from[1] == ' ') from++;
+				/* FALL-THROUGH */
+
+			default:
+				*(to++) = *(from++);
+			}
+		} /* end of string, or found a space */
+
+		*(to++) = '\0';	/* terminate the string */
+	}
+
+	/*
+	 *	We have to have SOMETHING, at least.
+	 */
+	if (argc <= 0) {
+		radlog(L_ERR, "rad_expand_xlat: Empty command line.");
+		return -1;
+	}
+
+	/*
+	 *	Expand each string, as appropriate.
+	 */
+	left = argv_buf + argv_buflen - to;
+	for (i = 0; i < argc; i++) {
+		int sublen;
+
+		/*
+		 *	Don't touch argv's which won't be translated.
+		 */
+		if (strchr(argv[i], '%') == NULL) continue;
+
+		if (!request) continue;
+
+		sublen = radius_xlat(to, left - 1, argv[i], request, NULL);
+		if (sublen <= 0) {
+			if (can_fail) {
+				/*
+				 *	Fail to be backwards compatible.
+				 *
+				 *	It's yucky, but it won't break anything,
+				 *	and it won't cause security problems.
+				 */
+				sublen = 0;
+			} else {
+				radlog(L_ERR, "rad_expand_xlat: xlat failed");
+				return -1;
+			}
+		}
+
+		argv[i] = to;
+		to += sublen;
+		*(to++) = '\0';
+		left -= sublen;
+		left--;
+
+		if (left <= 0) {
+			radlog(L_ERR, "rad_expand_xlat: Ran out of space while expanding arguments.");
+			return -1;
+		}
+	}
+	argv[argc] = NULL;
+
+	return argc;
+}
+
